"{ Package: 'stx:goodies/cypress' }"

CypressDefinition subclass:#CypressClassDefinition
	instanceVariableNames:'name superclassName category comment instVarNames
		classInstVarNames'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Definitions'
!

CypressClassDefinition comment:'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'
!


!CypressClassDefinition class methodsFor:'instance creation'!

name: aClassName 
superclassName: aSuperclassName
category: aCategory
instVarNames: anInstanceVariableNames
classInstVarNames: aClassInstanceVariableNames
comment: aComment

	^(self new) 
		name: aClassName 
		superclassName: aSuperclassName
		category: aCategory
		instVarNames: anInstanceVariableNames
		classInstVarNames: aClassInstanceVariableNames
		comment: aComment
! !

!CypressClassDefinition methodsFor:'accessing'!

category

	^category
!

classInstVarNames

	^classInstVarNames
!

className

	^self name
!

comment

	^comment
!

description

	^ Array with: name
!

instVarNames

	^instVarNames
!

name

	^name
!

superclassName

	^superclassName
! !

!CypressClassDefinition methodsFor:'comparing'!

= aDefinition
	^(super = aDefinition)
		and: [superclassName = aDefinition superclassName
		and: [category = aDefinition category
		and: [instVarNames = aDefinition instVarNames
		and: [classInstVarNames = aDefinition classInstVarNames
		and: [comment = aDefinition comment]]]]]
!

hash
    | hash |
    hash := String stringHash: name initialHash: 0.
    hash := String stringHash: superclassName initialHash: hash.
    hash := String stringHash: (category ifNil: [ '' ]) initialHash: hash.
    instVarNames , classInstVarNames do: [ :vName | hash := String stringHash: vName initialHash: hash ].
    ^ hash
! !

!CypressClassDefinition methodsFor:'converting'!

asCypressClassDefinition

	^self
! !

!CypressClassDefinition methodsFor:'dependency'!

provisions
	"Answer list of global names defined by this definition"

	^{ self name }
!

requirements
	"Answer list of global names required by this definition"

	^{self superclassName}
! !

!CypressClassDefinition methodsFor:'initialization'!

name: aClassName superclassName: aSuperclassName category: aCategory instVarNames: anInstanceVariableNames classInstVarNames: aClassInstanceVariableNames comment: aComment

	name := aClassName asSymbol.
	superclassName := aSuperclassName asSymbol.
	category := aCategory asSymbol.
	instVarNames := anInstanceVariableNames.
	classInstVarNames := aClassInstanceVariableNames.
	comment := aComment
! !

!CypressClassDefinition methodsFor:'loading'!

actualClass

	^Smalltalk at: self name
!

createClass

	| superClass |
	superClass := Smalltalk at: self superclassName.
	^ClassBuilder new
		superclass: superClass 
		subclass: self name
		instanceVariableNames: (self stringForVariables: self instVarNames)
		classVariableNames: '' poolDictionaries: '' category: self category
!

loadClassDefinition

	 | cls |
	cls := self createClass.
	cls class instanceVariableNames: (self stringForVariables: self classInstVarNames).
	self comment notEmpty ifTrue: [ cls comment: self comment ]
!

unloadDefinition

	Smalltalk removeClass: self actualClass.
! !

!CypressClassDefinition methodsFor:'printString'!

printString

	| str |
	str := WriteStream on: String new.
	str 
		nextPutAll: super printString;
		nextPutAll: ' (';
		nextPutAll: self name;
		nextPutAll: ')'.
	^str contents
!

stringForVariables: aCollectionOfVariableNames
	^ String streamContents:
		[:stream |
		aCollectionOfVariableNames
			do: [:ea | stream nextPutAll: ea]
			separatedBy: [stream space]]
! !

!CypressClassDefinition methodsFor:'visiting'!

classDefinition: classBlock methodDefinition: methodBlock

	classBlock value: self
! !

!CypressClassDefinition class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
