"{ Package: 'stx:goodies/cypress' }"

Object subclass:#CypressLoader
	instanceVariableNames:'additions removals unloadable provisions errors methodAdditions
		requirements'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Definitions'
!

CypressLoader comment:'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'
!


!CypressLoader class methodsFor:'loading'!

updatePackage: aPackage withSnapshot: aSnapshot
	self new
		updatePackage: aPackage withSnapshot: aSnapshot;
		load
! !

!CypressLoader methodsFor:'accessing'!

additions

	additions ifNil: [ additions := OrderedCollection new ].
	^additions
!

errors
	errors ifNil: [ errors := OrderedCollection new ].
	^errors
!

methodAdditions

	^#()
!

provisions
	^ provisions ifNil: [provisions := (Smalltalk classes collect: [:cl | cl name]) asSet ]
!

removals

	removals ifNil: [ removals := OrderedCollection new ].
	^removals
!

unloadable

	unloadable ifNil: [ unloadable := OrderedCollection new ].
	^unloadable
! !

!CypressLoader methodsFor:'applying'!

applyAddition: aCypressPatchOperation

	self additions add: aCypressPatchOperation
!

applyModification: aCypressPatchOperation

	self additions add: aCypressPatchOperation
!

applyRemoval: aCypressPatchOperation

	self removals add: aCypressPatchOperation
! !

!CypressLoader methodsFor:'error handling'!

handleErrorFor: aPatchOperation during: aBlock
	aBlock on: Error do: [:ex | self errors add: aPatchOperation ].
! !

!CypressLoader methodsFor:'loading'!

analyze

	self 
		analyzeAdditions;
		analyzeRemovals
!

analyzeAdditions

	| sorter |
	sorter := CypressDependencySorter new 
		addAll: self additions;
		addExternalProvisions: self provisions;
		yourself.
	additions := sorter orderedItems.
	requirements := sorter externalRequirements.
	unloadable := sorter itemsWithMissingRequirements.
!

analyzeRemovals

	| sorter |
	sorter := CypressDependencySorter new 
		addAll: self removals;
		yourself.
	removals := sorter orderedItems reversed.
!

basicLoad
	errors := OrderedCollection new.
	self additions do: [:ea | self loadClassDefinition: ea ]. "load class definitions first"
	self additions do: [:ea | self loadMethodDefinition: ea ] . "load method definitions now"
	self removals do: [:ea | self unloadDefinition: ea ]. "now we can remove things"
	self errors do: [:ea | ea addMethodAdditionTo: methodAdditions]. "not sure about methodAddtions...yet"
	self methodAdditions do: [:ea | self loadMethodAddition: ea ]. "ditto"
	self additions do: [:ea | self postLoad: ea ]. "this is where the obsoletion is taken into account ..."
!

load

	self analyze.
	self unloadable isEmpty ifFalse: [self unloadableDefinitionsError].
	self basicLoad
!

updatePackage: aPackage withSnapshot: aSnapshot
	|  patch snapshot |
	snapshot := aPackage snapshot.
	patch := aSnapshot patchRelativeToBase: snapshot.
	patch applyTo: self.
	snapshot definitions do: [:ea | self provisions addAll: ea provisions]
! !

!CypressLoader methodsFor:'operations'!

loadClassDefinition: aPatchOperation

	self 
		handleErrorFor: aPatchOperation 
		during: [ aPatchOperation loadClassDefinition ]
!

loadMethodDefinition: aPatchOperation
	
	self 
		handleErrorFor: aPatchOperation 
		during: [ aPatchOperation loadMethodDefinition ]
!

postLoad: aPatchOperation
	aPatchOperation postLoadDefinition
!

unloadDefinition: aPatchOperation
	
	self 
		handleErrorFor: aPatchOperation 
		during: [ aPatchOperation unloadDefinition ]
! !

!CypressLoader class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
