"{ Package: 'stx:goodies/cypress' }"

CypressDefinition subclass:#CypressMethodDefinition
	instanceVariableNames:'classIsMeta source category selector className timeStamp'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Definitions'
!

CypressMethodDefinition comment:'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'
!


!CypressMethodDefinition class methodsFor:'instance creation'!

className: aName
classIsMeta: isMetaclass
selector: aSelector
category: aCategory
source: aSource

	^(self new)
		className: aName
		classIsMeta: isMetaclass
		selector: aSelector
		category: aCategory
		source: aSource
!

className: aName
classIsMeta: isMetaclass
selector: aSelector
category: aCategory
source: aSource
timeStamp: aTimeStamp

	^(self new)
		className: aName
		classIsMeta: isMetaclass
		selector: aSelector
		category: aCategory
		source: aSource
		timeStamp: aTimeStamp
! !

!CypressMethodDefinition methodsFor:'accessing'!

category

	^category
!

classIsMeta

	^classIsMeta
!

className

	^className
!

description
	^ Array	
		with: className
		with: selector
		with: classIsMeta
!

selector

	^selector
!

source

	^source
!

timeStamp

	^timeStamp
! !

!CypressMethodDefinition methodsFor:'comparing'!

= aDefinition
    ^ super = aDefinition
        and: [ aDefinition source = self source
                and: [ aDefinition category = self category ] ]
!

hash
    | hash |
    hash := String stringHash: classIsMeta asString initialHash: 0.
    hash := String stringHash: source initialHash: hash.
    hash := String stringHash: category initialHash: hash.
    hash := String stringHash: className initialHash: hash.
    ^ hash
! !

!CypressMethodDefinition methodsFor:'converting'!

asCypressMethodDefinition

	^self
! !

!CypressMethodDefinition methodsFor:'dependency'!

requirements
	"Answer list of global names required by this definition"

	^{self className}
! !

!CypressMethodDefinition methodsFor:'initialization'!

className: aName classIsMeta: isMetaclass selector: aSelector category: aCategory source: aSource

	className := aName asSymbol.
	classIsMeta := isMetaclass.
	selector := aSelector asSymbol.
	category := aCategory asSymbol.
	source := aSource withLineEndings: String lfString.
!

className: aName classIsMeta: isMetaclass selector: aSelector category: aCategory source: aSource timeStamp: aTimeStamp

	className := aName asSymbol.
	classIsMeta := isMetaclass.
	selector := aSelector asSymbol.
	category := aCategory asSymbol.
	source := aSource withLineEndings: String lfString.
	timeStamp := aTimeStamp
! !

!CypressMethodDefinition methodsFor:'loading'!

actualClass

	| cls |
	cls := self theNonMetaClass.
	^self classIsMeta
		ifTrue: [ cls class ]
		ifFalse: [ cls  ].
!

loadMethodDefinition

	self actualClass
		compile: self source
		classified: self category
		withStamp: self timeStamp
		notifying: nil.
!

postLoadOver: aDefinition

	super postLoadOver: aDefinition.
	(self isInitializer
		and: [ aDefinition isNil or: [ self source ~= aDefinition source ]]) 
			ifTrue: [ self theNonMetaClass initialize ].
!

theNonMetaClass
	^Smalltalk at: self className
!

unloadDefinition

	self actualClass removeSelector: self selector asSymbol
! !

!CypressMethodDefinition methodsFor:'printing'!

printString

	| str |
	str := WriteStream on: String new.
	str 
		nextPutAll: super printString;
		nextPutAll: ' (';
		nextPutAll: self className.
	self classIsMeta
		ifTrue: [ str nextPutAll: ' class' ].
	str 
		nextPutAll: '>>';
		nextPutAll: self selector;
		nextPutAll: ')'.
	^str contents
! !

!CypressMethodDefinition methodsFor:'testing'!

isInitializer
	^ self selector = 'initialize' and: [self classIsMeta]
! !

!CypressMethodDefinition methodsFor:'visiting'!

classDefinition: classBlock methodDefinition: methodBlock

	methodBlock value: self
!

instanceMethod: instanceBlock classMethod: classBlock

	^(self classIsMeta
		ifTrue: [ classBlock ]
		ifFalse: [ instanceBlock ]) value: self
! !

!CypressMethodDefinition class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
