"{ Package: 'stx:goodies/cypress' }"

Object subclass:#CypressPackageReader
	instanceVariableNames:'packageDirectory packageStructure properties'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Structure'
!

CypressPackageReader comment:'Reader for the Cypress multi-dialect file format for Smalltalk packages

	CypressPackageReader installAsCodePackage: (FileDirectory default directoryNamed: ''Cypress-Mocks.package'')'
!


!CypressPackageReader class methodsFor:'instance creation'!

readPackageFrom: aPackagesDirectory

        ^(self new)
                packageDirectory: aPackagesDirectory;
                read;
                yourself

    "Created: / 30-08-2012 / 14:57:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageReader methodsFor:'accessing'!

packageDirectory

	^packageDirectory
!

packageDirectory: aDirectory

        packageDirectory := aDirectory asFilename

    "Modified: / 30-08-2012 / 14:59:24 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

packageStructure

	^packageStructure
!

packageStructure: aPackageStructure

	packageStructure := aPackageStructure
! !

!CypressPackageReader methodsFor:'converting'!

asChangeSet
    ^packageStructure asChangeSet

    "Created: / 31-08-2012 / 20:05:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageReader methodsFor:'private'!

classStructureFrom: classPropertiesDict 

	^(CypressClassStructure new)
		isClassExtension: true;
		properties: classPropertiesDict;
		packageStructure: packageStructure;
		yourself
!

classStructureFrom: classPropertiesDict comment: classComment.

	^(self classStructureFrom: classPropertiesDict)
		isClassExtension: false;
		comment: classComment;
		yourself
! !

!CypressPackageReader methodsFor:'reading'!

read

    	self readPropertiesFile.
	self readPackageStructure
!

readClassStructureFromEntry: classEntry
        | classDirectory methodPropertiesDict classPropertiesDict classComment entries classStructure propertyEntry |

        classDirectory := classEntry asFilename.
        entries := classDirectory directoryContentsAsFilenames.
        propertyEntry := entries detect: [ :entry | entry baseName = 'properties.json' ] ifNone: [ nil ] .
        propertyEntry notNil ifTrue:[
            propertyEntry readingFileDo: [ :fileStream | classPropertiesDict := CypressJsonParser parseStream: fileStream ].
        ].
        propertyEntry := entries detect: [ :entry | entry baseName = 'README.md' ] ifNone: [ nil ] .
        propertyEntry notNil ifTrue:[
            propertyEntry readingFileDo: [ :fileStream | classComment := fileStream contents ].
        ].

        classStructure := self
                classStructureFrom: classPropertiesDict
                comment: classComment.
        self
                readMethodStructureFor: classStructure
                in: entries
                methodProperties: methodPropertiesDict.
        ^ classStructure.

    "Modified: / 31-08-2012 / 19:54:10 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (format): / 03-09-2012 / 11:32:22 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

readExtensionClassStructureFromEntry: classEntry
        | classDirectory methodPropertiesDict classPropertiesDict entries classStructure propertyEntry |

        classDirectory := classEntry asFilename.
        entries := classDirectory directoryContentsAsFilenames.

        propertyEntry := entries detect: [ :entry | entry baseName = 'properties.json' ] ifNone: [ nil ] .
        propertyEntry notNil ifTrue:[
            propertyEntry readingFileDo: [ :fileStream | classPropertiesDict := CypressJsonParser parseStream: fileStream ].
        ].


        classStructure := self classStructureFrom: classPropertiesDict.
        self
                readMethodStructureFor: classStructure
                in: entries
                methodProperties: methodPropertiesDict.
        ^ classStructure

    "Modified: / 31-08-2012 / 19:55:49 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

readMethodStructureFor: classStructure in: entries methodProperties: methodPropertiesDict
        entries do: [ :entry | | methods isMeta |
                isMeta := false.
                methods := entry baseName = 'class'
                        ifTrue: [
                                isMeta := true.
                                classStructure classMethods ]
                        ifFalse: [ classStructure instanceMethods ].
                (entry baseName = 'instance' or: [ entry baseName = 'class' ]) ifTrue: [
                        (entry asFilename directoryContentsAsFilenames select: [ :each |
                                each name first ~= $. and: [ each name endsWith: '.st' ]]) do: [ :methodEntry |
                                methodEntry readingFileDo: [ :fileStream | | category source selector timeStamp |
                                        category := fileStream nextLine.
                                        source := fileStream upToEnd.
                                        selector := Parser new parseSelector: source.
                                        "
                                        timeStamp := (methodPropertiesDict
                                                at: (isMeta ifTrue: ['class'] ifFalse: ['instance']))
                                                at: selector.
                                        "
                                        methods
                                                at: selector
                                                put:
                                                        (CypressMethodStructure new
                                                                 classStructure: classStructure;
                                                                 name: selector;
                                                                 isMetaclass: isMeta;
                                                                 selector: selector;
                                                                 category: category;
                                                                 source: source;
                                                                 timeStamp: timeStamp
                                                                 yourself) ]]]]

    "Modified: / 31-08-2012 / 09:18:36 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

readPackageStructure
    packageStructure := CypressPackageStructure new.
    packageStructure name: (properties at: #'_stx_name') .

    self packageDirectory directoryContentsAsFilenamesDo: [ :entry |
            entry name first ~= $. ifTrue: [
                    (entry name endsWith: '.class') ifTrue: [
                            self packageStructure classes add: (self readClassStructureFromEntry: entry) ].
                    (entry name endsWith: '.extension') ifTrue: [
                            self packageStructure extensions add: (self readExtensionClassStructureFromEntry: entry) ]]]

    "Modified (format): / 31-08-2012 / 19:56:18 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

readPropertiesFile      

        (self packageDirectory asFilename  / 'properties.json')
                readingFileDo:[:fileStream |
                        properties := CypressJsonParser parseStream: fileStream ]

    "Modified: / 30-08-2012 / 14:58:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageReader class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
