"{ Package: 'stx:goodies/cypress' }"

Object subclass:#CypressPackageWriter
	instanceVariableNames:'packageStructure rootDirectory packageDirectory'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Structure'
!

CypressPackageWriter class instanceVariableNames:'specials'

"
 No other class instance variables are inherited by this class.
"
!


!CypressPackageWriter class methodsFor:'instance creation'!

writePackageStructure: aPackageStructure to: aPackagesDirectory

	self new
		packageStructure: aPackageStructure;
		rootDirectory: aPackagesDirectory;
		write
! !

!CypressPackageWriter class methodsFor:'as yet unclassified'!

initializeSpecials
    | map |
    map := Dictionary new.
    map
        at: $+ put: 'plus';
        at: $- put: 'minus';
        at: $= put: 'equals';
        at: $< put: 'less';
        at: $> put: 'more';
        at: $% put: 'percent';
        at: $& put: 'and';
        at: $| put: 'pipe';
        at: $* put: 'star';
        at: $/ put: 'slash';
        at: $\ put: 'backslash';
        at: $~ put: 'tilde';
        at: $? put: 'wat';
        at: $@ put: 'at'.
    map keys do: [ :key | map at: (map at: key) put: key ].
    ^ map
!

specials
    ^ specials ifNil: [ specials := self initializeSpecials ]
! !

!CypressPackageWriter class methodsFor:'services'!

writePackage: packageId to: directory
    "Writes a given package to a directory in Cypress format"

    CypressPackageWriter
        writePackageStructure: 
            (CypressPackageStructure fromPackage: 
                (CypressPackageDefinition new name: packageId))
        to: directory

    "Created: / 30-08-2012 / 14:29:17 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageWriter methodsFor:'accessing'!

packageDirectory

    packageDirectory isNil ifTrue: [ 
        packageDirectory := self rootDirectory asFilename / self packageStructure name.
        packageDirectory exists ifFalse:[
            packageDirectory recursiveMakeDirectory 
        ]
    ].
    ^packageDirectory

    "Modified: / 30-08-2012 / 14:38:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

packageDirectory: aPackageDirectory

	packageDirectory := aPackageDirectory
!

packageStructure

	^packageStructure
!

packageStructure: aCypressPackageStructure

	packageStructure := aCypressPackageStructure
!

rootDirectory

	^rootDirectory
!

rootDirectory: aDirectory

	rootDirectory := aDirectory
! !

!CypressPackageWriter methodsFor:'private'!

directoryForDirectoryNamed: directoryNameOrPath
    | directory |

    directoryNameOrPath = '.'
        ifTrue: [ directory := self packageDirectory ]
        ifFalse:[ directory := self packageDirectory / directoryNameOrPath ].
    directory exists ifFalse:[
        directory recursiveMakeDirectory
    ].
    ^directory

    "Modified: / 30-08-2012 / 14:48:12 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

fileNameForSelector: selector
    ^ selector last = $:
        ifTrue: [ 
            selector
                collect: [ :each | 
                    each = $:
                        ifTrue: [ $. ]
                        ifFalse: [ each ] ] ]
        ifFalse: [ 
            selector first isLetter
                ifTrue: [ selector ]
                ifFalse: [ 
                    | output specials |
                    specials := self class specials.
                    output := String new writeStream.
                    output nextPut: $^.
                    selector do: [ :each | output nextPutAll: (specials at: each) ] separatedBy: [ output nextPut: $. ].
                    output contents ] ]
!

writeInDirectoryName: directoryNameOrPath fileName: fileName extension: ext visit: visitBlock
    | directory |
    directory := self directoryForDirectoryNamed: directoryNameOrPath.
    (directory / (fileName , ext))
        writingFileDo:[:file| 
            visitBlock value: file 
        ]

    "Modified: / 30-08-2012 / 14:41:23 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageWriter methodsFor:'writing'!

write

        self packageDirectory exists
        ifTrue: [ self packageDirectory recursiveRemove. self packageDirectory makeDirectory ].
        self writePropertiesFile.
        self writePackageStructure

    "Modified: / 30-08-2012 / 14:39:02 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

writeClassComment: classStructure on: fileStream

    fileStream nextPutAll: (classStructure comment withLineEndings: String lfString)
!

writeClassStructure: classStructure on: fileStream

    | properties |
    properties := Dictionary new.
    properties at: 'name' put: classStructure className.
    properties at: 'super' put: classStructure superclassName.
    properties at: 'instvars' put: classStructure instanceVariableNames.
    properties at: 'classinstvars' put: classStructure classInstanceVariableNames.
    properties writeCypressJsonOn: fileStream forHtml: true indent: 0
!

writeClassStructure: classStructure to: classPath

    self
        writeInDirectoryName: classPath
        fileName: 'README'
        extension: '.md'
        visit: [:fileStream | self writeClassComment: classStructure on: fileStream ].
    self
        writeInDirectoryName: classPath
        fileName: 'properties'
        extension: '.json'
        visit: [:fileStream | self writeClassStructure: classStructure on: fileStream ]
!

writeExtensionClassStructure: classStructure to: classPath

     self
        writeInDirectoryName: classPath
        fileName: 'properties'
        extension: '.json'
        visit: [:fileStream |  | properties |
    		properties := Dictionary new.
    		properties at: 'name' put: classStructure className.
    		properties writeCypressJsonOn: fileStream forHtml: true indent: 0 ]
!

writeMethodStructure: methodStructure to:methodPath

    | filename |
    filename := self fileNameForSelector: methodStructure selector.
    self
        writeInDirectoryName: methodPath
        fileName: filename
        extension: '.st'
        visit: [:fileStream |
                fileStream
                        nextPutAll: methodStructure category;
                        cr;
                        nextPutAll: (methodStructure source withLineEndings: String lfString) ]

    "Modified: / 30-08-2012 / 14:49:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

writePackageStructure

	self writePackageStructureClasses:  self packageStructure classes isClassExtension: false.
	self writePackageStructureClasses:  self packageStructure extensions isClassExtension: true
!

writePackageStructureClasses: classStructures isClassExtension: isClassExtension
	| classDirExtension methodProperties classMethodsMap instanceMethodMap |
	classDirExtension _ isClassExtension
		ifTrue: [ '.extension' ]
		ifFalse: [ '.class' ].
	classStructures do: [ :classStructure | | classPath instanceMethodPath classMethodPath |
		classPath _ classStructure className , classDirExtension , FileDirectory slash.
		isClassExtension
			ifTrue: [
				self writeExtensionClassStructure: classStructure to: classPath ]
			ifFalse: [
				self writeClassStructure: classStructure to: classPath ].
		methodProperties _ Dictionary new.
		instanceMethodPath _ classPath , 'instance' , FileDirectory slash.
		methodProperties at: 'instance' put: (instanceMethodMap _ Dictionary new).
		classStructure instanceMethods do: [ :methodStructure |
			self writeMethodStructure: methodStructure to: instanceMethodPath.
			instanceMethodMap
				at: methodStructure selector asString
				put: methodStructure timeStamp ].
		classMethodPath _ classPath , 'class' , FileDirectory slash.
		methodProperties at: 'class' put: (classMethodsMap _ Dictionary new).
		classStructure classMethods do: [ :methodStructure |
			self writeMethodStructure: methodStructure to: classMethodPath.
			classMethodsMap
				at: methodStructure selector asString
				put: methodStructure timeStamp ].
		self
			writeInDirectoryName: classPath
			fileName: 'methodProperties'
			extension: '.json'
			visit: [ :fileStream | methodProperties writeCypressJsonOn: fileStream forHtml: false indent: 0 ]]
!

writePropertiesFile

    self
        writeInDirectoryName: '.'
        fileName: 'properties'
        extension: '.json'
        visit: [:fileStream | packageStructure properties writeCypressJsonOn: fileStream forHtml: true indent: 0 ]

    "Modified: / 30-08-2012 / 15:09:30 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressPackageWriter class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
