"{ Package: 'stx:goodies/cypress' }"

Object subclass:#CypressStructure
	instanceVariableNames:'name properties packageStructure'
	classVariableNames:''
	poolDictionaries:''
	category:'Cypress-Structure'
!

CypressStructure comment:'Main comment stating the purpose of this class and relevant relationship to other classes.

Possible useful expressions for doIt or printIt.

Structure:
 instVar1		type -- comment about the purpose of instVar1
 instVar2		type -- comment about the purpose of instVar2

Any further useful comments about the general approach of this implementation.'
!


!CypressStructure class methodsFor:'instance creation'!

fromJs: jsObject

	^(self new) 
		fromJs: jsObject asCypressPropertyObject;
		yourself
! !

!CypressStructure methodsFor:'accessing'!

name

	^name
!

name: aString 

	name := aString
!

packageStructure
	^packageStructure
!

packageStructure: aCypressPackageStructure
	packageStructure := aCypressPackageStructure
!

properties

	properties ifNil: [ properties := Dictionary new ].
	^properties
! !

!CypressStructure methodsFor:'converting'!

asChangeSet

    ^ChangeSet streamContents:[:changeStream | self changesOn: changeStream ]

    "Created: / 31-08-2012 / 20:01:07 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

changesOn: aStream

    self subclassResponsibility

    "Created: / 31-08-2012 / 20:02:05 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressStructure methodsFor:'initialization'!

fromJs: jsObject

	self subclassResponsibility
!

fromPackage: aCypressPackageDefinition

        | snapshot classMap classDefinitions classStructure |
        snapshot := aCypressPackageDefinition snapshot.
        name := ((aCypressPackageDefinition name 
                    copyReplaceAll: $: with:$_)
                    replaceAll: $/ with: $_)
                    , '.package'.
        properties  := Dictionary new.
        classDefinitions := Set new.
        classMap := Dictionary new.
        snapshot definitions do: [:definition |  
                        definition 
                                classDefinition: [:classDefinition |  classDefinitions add: classDefinition ] 
                                methodDefinition: [:methodDefinition | 
                                        (classMap 
                                                at: methodDefinition className 
                                                ifAbsent: [classMap at: methodDefinition className put: Set new]) 
                                                        add: methodDefinition. ]].
        classDefinitions do: [:classDefinition | 
                classStructure := (CypressClassStructure fromClassDefinition: classDefinition)
                        packageStructure: self.
                (classMap removeKey: classDefinition className ifAbsent: [#()]) do: [:methodDefinition | | methodStructure |
                        methodStructure := (CypressMethodStructure fromMethodDefinition: methodDefinition)
                                packageStructure: self;
                                classStructure: classStructure.
                        (methodDefinition
                                instanceMethod: [:instanceMethod | classStructure instanceMethods ] 
                                classMethod: [:classMethod | classStructure classMethods ])
                                        at: methodDefinition selector
                                        put: methodStructure ].
                self classes add: classStructure ].
        classMap keysAndValuesDo: [:className :methods |
                classStructure := (CypressClassStructure new name: className)
                        packageStructure: self.
                methods do: [:methodDefinition | | methodStructure |
                        methodStructure := (CypressMethodStructure fromMethodDefinition: methodDefinition)
                                packageStructure: self;
                                classStructure: classStructure.
                        (methodDefinition
                                instanceMethod: [:instanceMethod | classStructure instanceMethods ] 
                                classMethod: [:classMethod | classStructure classMethods ])
                                        at: methodDefinition selector
                                        put: methodStructure ].
                self extensions add: classStructure ].

    "Modified: / 30-08-2012 / 15:10:29 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!CypressStructure methodsFor:'writing'!

path: aFSPath file: aFilename write: writeBlock

	| fs stream |
	fs := aFSPath fs.
	stream := fs createWriteStream: (aFSPath resolve: aFilename).
	writeBlock value: stream.
	stream end.
!

writeJsonOn: aStream

	self writeJsonOn: aStream indent: 0.
!

writeJsonOn: aStream  indent: indent

	self subclassResponsibility
! !

!CypressStructure class methodsFor:'documentation'!

version_SVN
    ^ '$Id::                                                                                                                        $'
! !
